package de.lluni.javann.layers;

import org.ejml.simple.SimpleMatrix;

import java.util.function.Function;

/**
 * Layer to pass each output of every neuron from the previous layer through an activation function
 */
public class ActivationLayer extends Layer {
    Function<SimpleMatrix, SimpleMatrix> activation;
    Function<SimpleMatrix, SimpleMatrix> activationPrime;

    /**
     * Creates new activation layer with an activation function and its derivative from {@link de.lluni.javann.functions.ActivationFunctions}
     * @param activation activation function
     * @param activationPrime derivative of the activation function to compute gradient for backpropagation
     */
    public ActivationLayer(Function<SimpleMatrix, SimpleMatrix> activation, Function<SimpleMatrix, SimpleMatrix> activationPrime) {
        this.activation = activation;
        this.activationPrime = activationPrime;
    }

    @Override
    public SimpleMatrix forwardPropagation(SimpleMatrix input) {
        this.input = input;
        this.output = activation.apply(input);
        return this.output;
    }

    @Override
    public SimpleMatrix backwardPropagation(SimpleMatrix outputError, double learningRate) {
        return activationPrime.apply(this.input).elementMult(outputError);
    }
}
